"""Copyright 2021 Felipe Bastos Nunes

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import click

from flaskstarter import __version__

from flaskstarter.tools.requirements import add_support_to
from flaskstarter.tools.templating import get_template


@click.group()
@click.version_option(version=f"{__version__}")
def flaskstarter():
    """A program to start a Flask project under a modular structure."""


@flaskstarter.command()
@click.argument(
    "name",
)
def init(name: str):
    """Creates the project's source tree.

    Args:
        name (str): Project's main package's name to create. If it's a dot, will use current
                    directory as project's name.

    """
    # All directories and basic python files are created here
    click.echo("Creating project's source tree... ", nl=False)

    if name == ".":
        name = os.path.split(os.getcwd())[1]

    try:
        os.mkdir(os.path.join(os.getcwd(), name))
    except FileExistsError:
        click.echo(
            f'Project or module with name "{name}" already exists. Exiting.'
        )
        exit(0)

    os.makedirs(os.path.join(os.getcwd(), name, "ext"))
    ext_is_package = open(
        os.path.join(os.getcwd(), name, "ext", "__init__.py"), "w"
    )
    ext_is_package.close()
    os.makedirs(os.path.join(os.getcwd(), name, "blueprints"))
    blueprint_is_package = open(
        os.path.join(os.getcwd(), name, "ext", "__init__.py"), "w"
    )
    blueprint_is_package.close()
    app_is_package = open(os.path.join(os.getcwd(), name, "__init__.py"), "w")
    app_is_package.close()

    os.makedirs(os.path.join(os.getcwd(), name, "templates"))
    os.makedirs(os.path.join(os.getcwd(), name, "static"))

    os.makedirs(os.path.join(os.getcwd(), "instance", "uploads"))
    click.echo("Done!")

    click.echo("Creating first python scripts and configurations... ", nl=False)

    templates_and_dest = {
        "app.pyt": os.path.join(os.getcwd(), name, "app.py"),
        "views.pyt": os.path.join(os.getcwd(), name, "views.py"),
        "index.htmlt": os.path.join(
            os.getcwd(), name, "templates", "index.html"
        ),
        "configuration.pyt": os.path.join(
            os.getcwd(), name, "ext", "configuration.py"
        ),
        "settings.tomlt": os.path.join(
            os.getcwd(), "instance", "settings.toml"
        ),
        "manage.pyt": os.path.join(os.getcwd(), "manage.py"),
    }

    for template, destination in templates_and_dest.items():
        with open(destination, "w") as f:
            template_file = get_template(template)
            f.write(template_file.render(name=name))

    click.echo("Done!")

    # Requirements will help you do the basic startup of your virtualenv.
    add_support_to("Flask==2.0.2")
    add_support_to("dynaconf==3.1.7")
    add_support_to("toml")
    add_support_to(f"flaskstarter=={__version__}")

    click.echo("If you do have other requirements, feel free to customize it.")

    click.echo("Project created!")

    return 0


if __name__ == "__main__":
    flaskstarter(
        prog_name="flaskstarter"
    )  # pylint: disable=unexpected-keyword-arg



"""Copyright 2021 Felipe Bastos Nunes

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

import os
import subprocess
import click


def add_support_to(module):
    """Writes down the module name on project's requirements.txt.

    Warning: called only on project's creation time.

    Args:
        module (string): Library to add to requirements.txt.
    """
    try:
        requirements = open(os.path.join(os.getcwd(), "requirements.txt"), "a")
        click.echo(f"Adding {module} support... ", nl=False)
        requirements.write(f"{module}{os.linesep}")
        click.echo("Done!")
    except FileNotFoundError:
        click.echo(f"Couldn't create requirements.txt.")
    finally:
        requirements.close()


def install_requirements(name):
    """Installs the requirements.txt on destination project's virtual environment.

    Warning: MS Windows wasn't tested yet.

    Args:
        name (string): Project's name.
    """
    cmd = ""
    if os.name == "posix":
        cmd = f'. {os.path.join(os.getcwd(), name, ".venv", "bin", "activate")}; pip install -r {os.path.join(os.getcwd(), name, "requirements.txt")};'
    elif os.name == "nt":
        cmd = f'call {os.path.join(os.getcwd(), name, ".venv", "Scripts", "activate")}; pip install -r {os.path.join(os.getcwd(), name, "requirements.txt")};'
    subprocess.call(cmd, shell=True)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flaskstarter', 'flaskstarter.tools']

package_data = \
{'': ['*'], 'flaskstarter': ['templates/*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0', 'MarkupSafe>=2.0.1,<3.0.0', 'click>=8.0.1,<9.0.0']

entry_points = \
{'console_scripts': ['flaskstarter = flaskstarter.flaskstart_cli:flaskstarter']}

setup_kwargs = {
    'name': 'flaskstarter',
    'version': '0.5.2',
    'description': 'A Flask project start-up CLI to create Flask modular ready projects.',
    'long_description': "# flaskstarter\n\n![](https://img.shields.io/pypi/l/flaskstarter) ![](https://img.shields.io/pypi/v/flaskstarter) ![](https://img.shields.io/pypi/wheel/flaskstarter) \n\nA Flask project start-up CLI to create modular ready projects.\n\nFlaskstarter assumes you know about Flask microframework and its mechanics in a begginer level. It can be really helpful if you are still using monolithic aproach, and needs to start using a modular architecture.\n\nIt also assumes you are using Python 3.6+.\n\n> Flaskstarter recommends the use of a python virtual environment for project\n> to work safely and isolated from your systems binnaries. Activate it before\n> the next steps.\n\nTo install flaskstarter use the usual:\n\n`pip install flaskstarter`\n\nTo see its version:\n\n`flaskstarter --version`\n\nTo see its help:\n\n`flaskstarter --help`\n\nTo start a project, create its folder and:\n\n`flaskstarter init main_module_name`\n\nIf you do want to use the same project's root folder as main module name:\n\n`flaskstarter init .`\n\nTo see init's help:\n\n`flaskstarter init --help`\n\nNow, after project creation, you can  make full use of manage.py, a script\nwith a CLI that may help you to automate some tasks inside project tree.\n\nBy now you can create a blueprint structure by typing the bellow on project root:\n\n`$ python manage.py plug-blueprint [blueprint_name]`\n\nIf it will work as an API blueprint, that's enough. But maybe it is not and you want to use private templates related only to this blueprint. This is solved by adding a '-t' or '--templates' to the above command.\n\nAfter that, flaskstarter goes onto instance/settings.toml file to register the blueprint on it. There is an EXTENSIONS variable where you can list all the plugins to autoimport if you create any by hand. It uses factory design.\n\nMost of the times you are adding some extensions and middlewares to your app, so\nyou can add an empty skelleton to instanciate and plug to app by typing:\n\n`$ python manage.py plug-extension [name]`\n\nAs with blueprints, flaskstarter registers extensions on instance/settings.\n\nTo run your app you can use the bellow on project root:\n\n`$ python manage.py runserver`\n\nAsk manage.py for runserver help to see its options.\n\nNow it is possible to plug a database and a migration extensions to the project. For a first experience Flaskstarter is running with flask-sqlalchemy and flask-migrate. The templates that generate the kickoff database use sqlite and the simplest thing possible. You will be able to plug a database by running:\n\n`$ python manage.py plug-database`\n\nWhen plug-database is ran, the manage script will create the migrations folder as \nAlembic requires. Once it is created the following commands will be available.\n\nThis will generate a migration script with Example as message:\n\n`$ python manage.py db-migrate Example`\n\nThis upgrades the database:\n\n`$ python manage.py db-upgrade`\n\nIf anything undesirable happens, this will downgrade the database:\n\n`$ python manage.py db-downgrade`\n\nFor other Flask-Migrate commands, you can export FLASK_APP on your shell and use\nflask db (command) as its documentation guides.\n\n## What the project does for you\n\nIt creates project tree, a functional virtualenv on .venv, the init and routes files with a helloworld example and a manage.py script to run the project with the virtual enviroment created and attach blueprints to it. It now installs the requirements on project's .venv on POSIX systems. Feel free to change to poetry and pyproject.toml pattern.\n\nA word of warning: when commiting and pushing your project to versioning servers, remember to put instance folder into .gitignore, if not yet. And then remember to place it into deploy destination.\n\n## What the project does not do for you\n\nIt doesn't force you to use poetry or any other tool but flask, toml and dynaconf on the Flask project created.\n\n## Future\n\n- Add automated tests for development enviroment of flaskstarter\n- Work on a better architecture for the generated project\n- Start new extensions by manage.py creating an empty skeleton\n\n## How can you help on flaskstarter development?\n\nFeel free to clone it and send us pull requests! Remember to comment the decisions you make so they can be better reviewed.\n",
    'author': 'Felipe Bastos Nunes',
    'author_email': 'felipe.bastosn@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/felipebastos/flaskstart',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Copyright 2021 Felipe Bastos Nunes

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from jinja2 import Environment, PackageLoader, select_autoescape, Template

env = Environment(
    loader=PackageLoader("flaskstarter", "templates"),
    autoescape=select_autoescape("pyt", "htmlt", "tomlt"),
)


def get_template(name: str) -> Template:
    """Retrieves and return the specified template.

    Args:
        name (str): Template's name

    Returns:
        Template: A Jinja2 Template object.
    """
    return env.get_template(name)
